import requests

from .constants import AIRPOL_URL

import json

from typing import Optional

from .apmodel import AirpolData

import os

API_KEY = os.getenv("API_KEY") or None

if not API_KEY:
    raise Exception("API_KEY doesn't exit")


def by_geoc(lat: float, lon: float, types: Optional[str] = "current") -> AirpolData:
    """
    Description
    -----------
    For Location coodinates returns:

    'AirpolData'

    Parameters
    ----------
    latitude : float
    longitude : float
    types : str, optional
        For value 'forecast' return forecast of next 5 days (default
        is current)

    Returns
    -------
    AirpolData
    """

    payload = {"lat": f"{lat}", "lon": f"{lon}", "appid": API_KEY}
    if types == "current":
        URL = f"{AIRPOL_URL}"
    elif types == "forecast":
        URL = f"{AIRPOL_URL}/{types.lower()}"
    else:
        raise Exception("types can either be current(default) or forecast")

    req = requests.get(URL, params=payload)
    if req.status_code == 200:
        return AirpolData.parse_raw(json.dumps(req.json()))
    else:
        return req.json()



import os

import json

import requests

from typing import Optional

from .constants import WEATH_URL

from .cwmodel import WeatherData

API_KEY = os.getenv("API_KEY") or None

if not API_KEY:
    raise Exception("API_KEY doesn't exit")


def by_cname(
    cityname: str,
    countrycode: Optional[str] = "",
    units: Optional[str] = "",
    lang: Optional[str] = "en",
) -> WeatherData.parse_raw:
    """
    Description
    -----------
    for a cityname, countrycode returns:

    'WeatherData'

    Parameters
    ----------
    cityname : str
        The name of the city or town
    countrycode : str, Optional
    units : str, Optional
        Temperature is available in Fahrenheit, Celsius and Kelvin units.
            For temperature in Fahrenheit use units=imperial
            For temperature in Celsius use units=metric
            Temperature in Kelvin is used by default
        Wind speed and gust is available in meter/sec, miles/hour.
            For meter/sec use units=metric
            For miles/hour use units=imperial
            meter/sec by default
    lang : str, Optional
        Language the information will be returned

    Returns
    -------
    WeatherData
    """
    payload = {
        "q": f"{cityname},{countrycode}",
        "units": f"{units}",
        "lang": f"{lang}",
        "appid": f"{API_KEY}",
    }
    req = requests.get(WEATH_URL, params=payload)
    if req.status_code == 200:
        return WeatherData.parse_raw(json.dumps(req.json()))
    else:
        return req.json()


def by_cid(
    cityid: int, units: Optional[str] = "", lang: Optional[str] = "en"
) -> WeatherData.parse_raw:
    """
    Description
    -----------
    for a cityid returns:

    'WeatherData'

    Parameters
    ----------
    cityid : int
        The id of the city
    units : str, Optional
        Temperature is available in Fahrenheit, Celsius and Kelvin units.
            For temperature in Fahrenheit use units=imperial
            For temperature in Celsius use units=metric
            Temperature in Kelvin is used by default
        Wind speed and gust is available in meter/sec, miles/hour.
            For meter/sec use units=metric
            For miles/hour use units=imperial
            meter/sec by default
    lang : str, Optional
        Language the information will be returned

    Returns
    -------
    WeatherData
    """
    payload = {
        "id": f"{cityid}",
        "units": f"{units}",
        "lang": f"{lang}",
        "appid": f"{API_KEY}",
    }
    req = requests.get(WEATH_URL, params=payload)
    if req.status_code == 200:
        return WeatherData.parse_raw(json.dumps(req.json()))
    else:
        return req.json()


def by_geoc(
    lat: float, lon: float, units: Optional[str] = "", lang: Optional[str] = "en"
) -> WeatherData.parse_raw:
    """
    Description
    -----------
    for a latitude, longitude returns:

    'WeatherData'

    Parameters
    ----------
    latitude : float
    longitude : float
    units : str, Optional
        Temperature is available in Fahrenheit, Celsius and Kelvin units.
            For temperature in Fahrenheit use units=imperial
            For temperature in Celsius use units=metric
            Temperature in Kelvin is used by default
        Wind speed and gust is available in meter/sec, miles/hour.
            For meter/sec use units=metric
            For miles/hour use units=imperial
            meter/sec by default
    lang : str, Optional
        Language the information will be returned

    Returns
    -------
    WeatherData
    """
    payload = {
        "lat": f"{lat}",
        "lon": f"{lon}",
        "units": f"{units}",
        "lang": f"{lang}",
        "appid": f"{API_KEY}",
    }
    req = requests.get(WEATH_URL, params=payload)
    if req.status_code == 200:
        return WeatherData.parse_raw(json.dumps(req.json()))
    else:
        return req.json()


def by_zcode(
    zipcode: int,
    countrycode: str,
    units: Optional[str] = "",
    lang: Optional[str] = "en",
) -> WeatherData.parse_raw:
    """
    Description
    -----------
    for a zipcode, countrycode returns:

    'WeatherData'

    Parameters
    ----------
    zipcode : int
    countrycode : str
        The code of the country
    units : str, Optional
        Temperature is available in Fahrenheit, Celsius and Kelvin units.
            For temperature in Fahrenheit use units=imperial
            For temperature in Celsius use units=metric
            Temperature in Kelvin is used by default
        Wind speed and gust is available in meter/sec, miles/hour.
            For meter/sec use units=metric
            For miles/hour use units=imperial
            meter/sec by default
    lang : str, Optional
        Language the information will be returned

    Returns
    -------
    WeatherData
    """
    payload = {
        "zip": f"{zipcode},{countrycode}",
        "units": f"{units}",
        "lang": f"{lang}",
        "appid": f"{API_KEY}",
    }
    req = requests.get(WEATH_URL, params=payload)
    if req.status_code == 200:
        return WeatherData.parse_raw(json.dumps(req.json()))
    else:
        return req.json()



from __future__ import annotations

from typing import List, Dict

from pydantic import BaseModel, Field

import orjson


def orjson_dumps(v, *, default):
    # orjson.dumps returns bytes, to match standard json.dumps we need to decode
    return orjson.dumps(v, default=default).decode()


class Main(BaseModel):
    aqi: int = Field(
        None,
        description="Air Quality Index Possible values: 1, 2, 3, 4, 5. Where 1 = Good, 2 = Fair, 3 = Moderate, 4 = Poor, 5 = Very Poor.",
    )


class Components(BaseModel):
    co: float = Field(None, description="Сoncentration of CO (Carbon monoxide), μg/m3")
    no: float = Field(
        None, description="Сoncentration of NO (Nitrogen monoxide), μg/m3"
    )
    no2: float = Field(
        None, description="Сoncentration of NO2 (Nitrogen dioxide), μg/m3"
    )
    o3: float = Field(None, description="Сoncentration of O3 (Ozone), μg/m3")
    so2: float = Field(
        None, description="Сoncentration of SO2 (Sulphur dioxide), μg/m3"
    )
    pm2_5: float = Field(
        None, description="Сoncentration of PM2.5 (Fine particles matter), μg/m3"
    )
    pm10: float = Field(
        None, description="Сoncentration of PM10 (Coarse particulate matter), μg/m3"
    )
    nh3: float = Field(None, description="Сoncentration of NH3 (Ammonia), μg/m3")


class ListItem(BaseModel):
    dt: int = Field(None, description="Date and time, Unix, UTC")
    main: Main
    components: Components


class AirpolData(BaseModel):
    coord: Dict[str, float] = Field(
        None,
        description="Coordinates from the specified location (latitude, longitude)",
    )
    list: List[ListItem]

    class Config:
        json_loads = orjson.loads
        json_dumps = orjson_dumps



from setuptools import setup,find_packages
with open("README.md", "r") as fh:
  long_description = fh.read()
setup(
  name = 'py-open-weather',         # How you named your package folder (MyLib)
  packages=find_packages(),
  include_package_data=True,
  version = '0.1.1',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'A python3 library providing information of Current & Forecast of Weather as well as Air Pollution based on data from OpenWeatherMap.',
  long_description = long_description,   # Give a long description about your library
  long_description_content_type = "text/markdown",
  author = 'CodewithRv',                   # Type in your name
  author_email = 'ravigoel.1997@gmail.com',      # Type in your E-Mail
  url = 'https://github.com/ravigoel08/py-open-weather',   # Provide either the link to your github or to your website
  download_url = '',    # I explain this later on
  keywords = ['open weather api', 'owm', 'weather','weather api', 'pyweather', 'open weather', 'air pollution', 'py-open-weather'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'requests',
          'pydantic',
          'orjson',
      ],
  classifiers=[
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.9',
    "Operating System :: OS Independent",
  ],
)


from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import orjson


def orjson_dumps(v, *, default):
    # orjson.dumps returns bytes, to match standard json.dumps we need to decode
    return orjson.dumps(v, default=default).decode()


class Coord(BaseModel):
    lon: float = Field(None, description="City geo location, longitude")
    lat: float = Field(None, description="City geo location, latitude")


class WeatherItem(BaseModel):
    id: int = Field(None, description="Weather condition id.")
    main: str = Field(
        None, description="Group of weather parameters (Rain, Snow, Extreme etc.)"
    )
    description: str = Field(None, description="Weather condition within the group.")
    icon: str = Field(None, description="Weather icon id.")


class Main(BaseModel):
    temp: float = Field(None, description="Temperature.")
    feels_like: float = Field(
        None,
        description="Temperature. This temperature parameter accounts for the human perception of weather.",
    )
    temp_min: float = Field(
        None,
        description="Minimum temperature at the moment. This is minimal currently observed temperature.",
    )
    temp_max: float = Field(
        None,
        description="Maximum temperature at the moment. This is maximal currently observed temperature.",
    )
    pressure: int = Field(
        None,
        description="Atmospheric pressure (on the sea level, if there is no sea_level or grnd_level data), hPa",
    )
    humidity: int = Field(None, description="Humidity, %")
    sea_level: Optional[int] = Field(
        None, description="Atmospheric pressure on the sea level, hPa"
    )
    grnd_level: Optional[int] = Field(
        None, description="Atmospheric pressure on the ground level, hPa"
    )


class Wind(BaseModel):
    speed: float = Field(None, description="Wind speed.")
    deg: int = Field(None, description="Wind direction, degrees (meteorological)")
    gust: Optional[int] = Field(None, description="Wind gust.")


class Clouds(BaseModel):
    all: int = Field(None, description="Cloudiness, %")


class Sys(BaseModel):
    type: int = Field(None, description="Internal parameter")
    id: int = Field(None, description="Internal parameter")
    message: float = Field(None, description="Internal parameter")
    country: str = Field(None, description="Country code (GB, JP etc.)")
    sunrise: int = Field(None, description="Sunrise time, unix, UTC ")
    sunset: int = Field(None, description="Sunset time, unix, UTC")


class WeatherData(BaseModel):
    coord: Coord
    weather: List[WeatherItem]
    base: str = Field(None, description="Internal parameter")
    main: Main
    visibility: int = Field(None, description="")
    wind: Wind
    clouds: Clouds
    dt: int = Field(None, description="Time of data calculation, unix, UTC")
    sys: Sys
    timezone: int = Field(None, description="Shift in seconds from UTC")
    id: int = Field(None, description="City ID")
    name: str = Field(None, description="City name")
    cod: int = Field(None, description="Internal parameter")

    class Config:
        json_loads = orjson.loads
        json_dumps = orjson_dumps
